@@ -3,28 +3,21 @@ import collections.abc as collections_abc
 import errno
 import hashlib
 import io
+import itertools
 import json
 import os
 import select
 import socket
 import ssl
 from datetime import datetime, timedelta
+from json.decoder import JSONDecodeError
 
 import decorator
 import urllib3
 from urllib3.util.ssl_ import ssl_wrap_socket as urllib3_ssl_wrap_socket
 from urllib3.util.ssl_ import wrap_socket as urllib3_wrap_socket
 
-from .compat import (
-    BlockingIOError,
-    FileNotFoundError,
-    JSONDecodeError,
-    basestring,
-    byte_type,
-    decode_from_bytes,
-    encode_to_bytes,
-    text_type,
-)
+from .compat import basestring, byte_type, decode_from_bytes, encode_to_bytes, text_type
 from .utils import SSL_PROTOCOL, MocketSocketCore, hexdump, hexload, wrap_ssl_socket
 
 xxh32 = None
@@ -140,7 +133,6 @@ class MocketSocket(object):
         self, family=socket.AF_INET, type=socket.SOCK_STREAM, proto=0, *args, **kwargs
     ):
         self.true_socket = true_socket(family, type, proto)
-        self._connected = False
         self._buflen = 65536
         self._entry = None
         self.family = int(family)
@@ -167,6 +159,12 @@ class MocketSocket(object):
             self.__class__.__name__, self.family, self.type, self.proto
         )
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
     @property
     def fd(self):
         if self._fd is None:
@@ -385,6 +383,8 @@ class MocketSocket(object):
         return len(data)
 
     def close(self):
+        if self.true_socket and not self.true_socket._closed:
+            self.true_socket.close()
         self._fd = None
 
     def __getattr__(self, name):
@@ -519,6 +519,13 @@ class Mocket(object):
     def get_truesocket_recording_dir(cls):
         return cls._truesocket_recording_dir
 
+    @classmethod
+    def assert_fail_if_entries_not_served(cls):
+        """ Mocket checks that all entries have been served at least once. """
+        assert all(
+            entry._served for entry in itertools.chain(*cls._entries.values())
+        ), "Some Mocket entries have not been served"
+
 
 class MocketEntry(object):
     class Response(byte_type):
@@ -528,8 +535,11 @@ class MocketEntry(object):
 
     request_cls = str
     response_cls = Response
+    responses = None
+    _served = None
 
     def __init__(self, location, responses):
+        self._served = False
         self.location = location
         self.response_index = 0
 
@@ -538,7 +548,7 @@ class MocketEntry(object):
         ):
             responses = [responses]
 
-        lresponses = []
+        self.responses = []
         for r in responses:
             if isinstance(r, BaseException):
                 pass
@@ -546,11 +556,10 @@ class MocketEntry(object):
                 if isinstance(r, text_type):
                     r = encode_to_bytes(r)
                 r = self.response_cls(r)
-            lresponses.append(r)
+            self.responses.append(r)
         else:
             if not responses:
-                lresponses = [self.response_cls(encode_to_bytes(""))]
-        self.responses = lresponses
+                self.responses = [self.response_cls(encode_to_bytes(""))]
 
     def can_handle(self, data):
         return True
@@ -564,6 +573,8 @@ class MocketEntry(object):
         if self.response_index < len(self.responses) - 1:
             self.response_index += 1
 
+        self._served = True
+
         if isinstance(response, BaseException):
             raise response
 
@@ -595,8 +606,8 @@ class Mocketizer(object):
         if callable(method):
             method()
 
-    @staticmethod
-    def wrap(test=None, truesocket_recording_dir=None):
+    @classmethod
+    def wrap(cls, test=None, truesocket_recording_dir=None):
         def wrapper(t, *args, **kw):
             instance = args[0] if args else None
             namespace = None
@@ -608,7 +619,7 @@ class Mocketizer(object):
                         t.__name__,
                     )
                 )
-            with Mocketizer(
+            with cls(
                 instance,
                 namespace=namespace,
                 truesocket_recording_dir=truesocket_recording_dir,
